/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#ifndef cmatrix_h
#define cmatrix_h

#include "linutil.h"
#include "complex.h"

/* cmatrix class characterizes matrices of complex values
   The "mat" member is a doubly indexed array of "complex" structs
   and mat[row][column] corresponds to the element in the specified
   row and column. The cmatrix object also knows about the width
   and height of the matrix
*/
typedef struct cmatrix
{
  complex **mat; /* Doubly indexed pointer to 2D array of complex values*/
  int width;	 /* width of complex matrix */
  int height;	 /* height of complex matrix */
} cmatrix;

/* Note that member functions all use the suffix CMatrix to
   distinguish that they act upon cmatrix objects
*/
   
#ifdef GCC_PROTOTYPES
cmatrix *zerosCMatrix(int w, int h);
/* returns	a newly allocated complex matrix of size width,height
   		where all of the values have been initialized to 0
 */
void freeCMatrix(cmatrix *);
/* effect	deallocates complex matrix referenced by given pointer 
 */

int containsCMatrix(cmatrix *, complex *val);
/* returns	a boolean characterizing whether the complex matrix
   		contains the given vector
 */

int isZeroCMatrix(cmatrix *);
/* returns	a boolean characterizing whether the complex matrix
   		is all zeros
 */

#else
cmatrix *zerosCMatrix();
void freeCMatrix();
int containsCMatrix();
int isZeroCMatrix();
#endif

#endif /* cmatrix_h */
