/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#include "complex.h"
#include "linutil.h"
#include <math.h>

complex **zerosComplexArray(w,h)
int w,h;
{
  complex **ans;
  int i;
  ans = (complex **)malloc(w*h*sizeof(complex *));

  for (i = 0; i < h ; ++i)
    {
      ans[i] = (complex *)malloc(w*sizeof(complex));
      memset(ans[i],0,w*sizeof(complex));
    }
  
  return ans;
}

void freeComplexArray(M,h)
complex **M;
int h;
{
  int i;

  if (!M) return;

  for (i = 0; i < h; ++i)
    free(M[i]), M[i] = NULL;
  free(M);
}

void printComplexArray(M,w,h)
complex **M;
int w;
int h;
{
  int i,j;
  
  for (i = 0; i < h; ++i)
    {
      printf(" [ ");

      for (j = 0; j < w; ++j)
	printf("%12.6f + %12.6f i ",M[i][j].real,M[i][j].imag);
      
      printf(" ]\n");
    }
}

void convertComplexArray(L,M,w,h)
complex *L,**M;
int w,h;
{
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      M[i][j] = L[i*w+j];
}

complex oneOverComplex(complex val)
{
  complex ans;
  double mag;
  mag = val.real*val.real+val.imag*val.imag;
  ans.real = val.real/mag;
  ans.imag = -val.imag/mag;
  return ans;
}

complex scaleComplex(complex val, double scale)
{
  complex ans;
  ans.real = val.real*scale;
  ans.imag = val.imag*scale;
  return ans;
}

complex addComplex(complex val1, complex val2)
{
  complex ans;
  ans.real = val1.real+val2.real;
  ans.imag = val1.imag+val2.imag;
  return ans;
}

complex multiplyComplex(complex val1, complex val2)
{
  complex ans;
  ans.real = val1.real*val2.real - val1.imag*val2.imag;
  ans.imag = val1.real*val2.imag + val2.real*val1.imag;
  return ans;
}

complex divideComplex(complex val1, complex val2)
{
  complex ans,recip;
  recip = oneOverComplex(val2);
  return multiplyComplex(val1,recip);
}

int isZeroComplex(complex val)
{
  return (!val.real && !val.imag);
}

int isRealComplex(complex val)
{
  double mag;
  mag = 1;
  if (fabs(val.real) > mag)
    mag = fabs(val.real);
  
  return (fabs(val.imag) < mag*1e-6);
}

complex mycpower(complex val, double exponent)
{
  complex ans;
  double ang;
  double mag;
  double newang;
  double newmag;
  
  ang = atan2(val.imag,val.real);
  mag = sqrt(val.real*val.real+val.imag*val.imag);
  newang = ang*exponent;
  newmag = pow(mag,exponent);
  
  ans.real = newmag*cos(newang);
  ans.imag = newmag*sin(newang);
  return ans;
}

int aequal(complex val1, complex val2)
{
  double mag = 1.;

  if (fabs(val1.real) > mag)
    mag = fabs(val1.real);

  if (fabs(val1.imag) > mag)
    mag = fabs(val1.imag);

  if (fabs(val2.real) > mag)
    mag = fabs(val2.real);

  if (fabs(val2.imag) > mag)
    mag = fabs(val2.imag);

  mag = mag * 1e-6;

  return (fabs(val1.real - val2.real) < mag &&
	  fabs(val1.imag - val2.imag) < mag);
}
