/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#ifndef complex_h
#define complex_h

/* complex class characterizes complex values with real and imaginary
   components.
*/

typedef struct complex
{ 
  double real; /* real component */
  double imag; /* imaginary component */
} complex;

/* Note that member functions all use the suffix Complex to
   distinguish that they act upon complex objects
*/

#ifdef GCC_PROTOTYPES
complex **zerosComplexArray(int w, int h);
/* returns	a pointer to a doubly indexed array of complex objects
   		where the doubly indexed array oges from [0..h][0..w]
   		and where all of the values have been initialized to 0
 */

complex oneOverComplex(complex val);
/* returns	a complex object corresponding to the geometrical
   		inverse of the given complex value  
 */

complex addComplex(complex val1, complex val2);
/* returns	a complex object corresponding to the sum of the two
   		given complex values 
 */

complex scaleComplex(complex val, double scale);
/* returns	a complex object corresponding to the scaling the 
   		given complex value by the given scale
*/

complex multiplyComplex(complex val1, complex val2);
/* returns	a complex object corresponding to the product of the two
   		given complex values 
 */

complex divideComplex(complex val1, complex val2);
/* returns	a complex object corresponding to the ratio of the two
   		given complex values 
 */

int isZeroComplex(complex val);
/* returns 	a boolean corresponding to whether the given complex
   		object equals 0 + 0 i
*/

int isRealComplex(complex val);
/* returns 	a boolean corresponding to whether the given complex
   		object is completely real (i.e. val.imag == 0)
*/

complex mycpower(complex val, double exponent);
/* returns 	a complex object corresponding to raising the given complex
   		value to the given exponent
*/

int aequal(complex val1, complex val2);
/* returns 	boolean characterizing whether the two complex values 
   		are approximately equal
*/

#else
complex **zerosComplexArray();
complex oneOverComplex();
complex addComplex();
complex scaleComplex();
complex divideComplex();
complex mycpower();
#endif
#endif /* complex_h */
