/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */


#include "linutil.h"
#include "compmat.h"
#include "cmatrix.h"
#include "condnum.h"
#include <stdlib.h>

int isLinearCompanionMatrix(matPoly)
matrixPolynomial *matPoly;
{
  int i,j,uDim;
  
  for (i = 0; i < matPoly->size; ++i)
    if (matPoly->matPoly[1]->mat[i][i] != 1)
      break;
  
  uDim = i;
  
  for (i = 0; i < uDim; ++i)
    for (j = 0; j < uDim; ++j)
      if (i != j)
	if (matPoly->matPoly[1]->mat[i][j])
	  return FALSE;

  return TRUE;
}

void companionMatrixLinear(matPoly, 
			   companionMatrix, 
			   process,
			   procvals,
			   debug)
matrixPolynomial *matPoly;
matrix *companionMatrix;
int *process;
double *procvals;
int debug;
{
  int i,uDim;
  matrix *tmp, *tArray, *pArray, *qArray, *rArray, *invPArray;
  matrix *m1,*m2;
  
  for (i = 0; i < matPoly->size; ++i)
    if (matPoly->matPoly[1]->mat[i][i] != 1)
      break;
  
  uDim = i;
  
  tmp = matPoly->matPoly[0];

#ifdef DEBUG
  printf("tmp\n");printMatrix(tmp);
#endif
  tArray = subMatrix(tmp,0,0,0,uDim,uDim);
#ifdef DEBUG
  printf("tArray\n");printMatrix(tArray);
#endif
  pArray =
    subMatrix(tmp,0,uDim,uDim,matPoly->size-uDim,matPoly->size-uDim);
#ifdef DEBUG
  printf("pArray\n");printMatrix(pArray);
#endif
  rArray = subMatrix(tmp,0,uDim,0,matPoly->size-uDim,uDim);
#ifdef DEBUG
  printf("rArray\n");printMatrix(rArray);
#endif
  qArray = subMatrix(tmp,0,0,uDim,uDim,matPoly->size-uDim);
#ifdef DEBUG
  printf("qArray\n");printMatrix(qArray);
#endif
  invPArray = inverseMatrix (pArray, 0);
#ifdef DEBUG
  printf("invPArray\n");printMatrix(invPArray);
#endif
  m1 = multiplyMatrix(invPArray,qArray,0);
#ifdef DEBUG
  printf("m1\n");printMatrix(m1);
#endif
  m2 = multiplyMatrix(rArray,m1,0);
#ifdef DEBUG
  printf("m2\n");printMatrix(m2);
#endif
  resizeMatrix(companionMatrix,uDim,uDim);

  subtractMatrix(tArray,m2,companionMatrix);
#ifdef DEBUG
  printf("companionMatrix\n");printMatrix(companionMatrix);
#endif
  *process = PROCESS_IDENTITY;
  procvals[0] = procvals[1] = procvals[2] = procvals[3] = 0;
  
  freeMatrix(m1);
  freeMatrix(m2);
  freeMatrix(invPArray);
  freeMatrix(pArray);
  freeMatrix(qArray);
  freeMatrix(rArray);
  freeMatrix(tArray);
}

int nChooseK(n,k)
int n, k;
{
  int i;
  int ans = 1;
  if (k == 0 || k == n)
    return ans;
  for (i = n; i >= n-k+1; --i)
    ans = ans * i;
  for (i = 1; i <= k; ++i)
    ans = ans / i;

  return ans;
}

double rationaltrans(alpha,beta,gamma,delta,maxdeg,j,i)
double alpha,beta,gamma,delta;
int maxdeg,i,j;
{
  double sum = 0;
  int mid,left,midleft;
  double part1,part2;

  left = maxdeg - i;
  for (mid = 0; mid <= i; ++mid)
    {
      double powA,powB,powC,powD;
      if (mid > j) continue;
      powA = pow(alpha,mid);
      powB = pow(beta,i-mid);
      part1 = powA * powB * nChooseK(i,mid);
      midleft = j-mid;
      if (midleft > left)
	continue;
      part2 = pow(gamma,midleft) * pow(delta,left-midleft)*
	nChooseK(left,midleft);
#ifdef DEBUG
      printf("mid %d part1 %f midleft %d left % d part2 %f powA %f powB %f\n",
	     mid,part1,midleft,left,part2,powA,powB);
#endif
      sum += part1*part2;
    }
  return sum;
}

int testrationaltrans()
{
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,4,4));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,4,3));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,4,2));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,4,1));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,4,0));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,3,4));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,3,3));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,3,2));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,3,1));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,3,0));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,2,4));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,2,3));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,2,2));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,2,1));
  printf("rat - %f\n",
	 rationaltrans(0.45,0.36,0.57,0.89,4,2,0));

}
  
void companionMatrix(matPoly, 
		     companionMatrix, 
		     process,
		     procvals,
		     debug)
matrixPolynomial *matPoly;
matrix *companionMatrix;
int *process;
double *procvals;
int debug;
{
  int i,j;
  int maxdeg,matsize,bigsize;
  matrix *leadingmatrix;
  matrix *invleadingmatrix = NULL;
  matrix *followingmatrix;
  matrix *matrixmult = NULL;
  matrix *window = NULL;
  double condleadingmatrix;
  double condcompanionMatrix;
  matrix *trailingmatrix;
  matrix *invtrailingmatrix;
  double condtrailingmatrix;
  double smallestcond;
  double alpha,beta,gamma,delta;
  double bestalpha,bestbeta,bestgamma,bestdelta;
  double rationalfactor;
  matrix *scalematrix;
  
  if (debug & 1)
    {
      printf("companionMatrix called with :\n");
      printMatrixPolynomial(matPoly);
    }
 
  maxdeg = matPoly->degree;

  if (maxdeg == 1)
    if (isLinearCompanionMatrix(matPoly))
      return companionMatrixLinear
	(matPoly,companionMatrix,process,procvals,debug);

  leadingmatrix = matPoly->matPoly[maxdeg];

  matsize = leadingmatrix->width;

  bigsize = maxdeg * matsize;

  if (debug & 4)
    {
      printf("matsize %d\n",matsize);
      printf("maxdeg %d\n",maxdeg);
      printf("leadingmatrix\n");
      printMatrix(leadingmatrix);
      printf("bigsize %d\n",bigsize);
    }
  
  resizeMatrix(companionMatrix,bigsize,bigsize);

  for (i = 0; i < bigsize; ++i)
    for (j = 0; j < bigsize; ++j)
      companionMatrix->mat[i][j] = 0;

  for (i = 0; i < bigsize-matsize; ++i)
    companionMatrix->mat[i][i+matsize] = 1;
  
  *process = PROCESS_UNKNOWN;
  procvals[0] = procvals[1] = procvals[2] = procvals[3] = 0;
  
  condleadingmatrix = Condition_Number_(leadingmatrix);

  if (debug & 4)
    {
      printf("cond(leading matrix) %f\n",condleadingmatrix);
    }
  
  if (condleadingmatrix < 1e6 && !(debug & 0x100))
    {
      if (debug & 12)
	{
	  printf("cond(leading matrix) < 1e6 && process == PROCESS_UNKNOWN\n");
	  printf("-> identity process\n");
	}
      
      *process = PROCESS_IDENTITY;
      
      invleadingmatrix = inverseMatrix(leadingmatrix,0);
      
      if (debug & 12)
	{
	  printf("invleadingmatrix\n");
	  printMatrix(invleadingmatrix);
	}
      
      for (i = 0; i < maxdeg; ++i)
	{
	  followingmatrix = matPoly->matPoly[i];

	  matrixmult =
	    multiplyMatrix(invleadingmatrix,followingmatrix,0);

	  if (debug & 12)
	    {
	      printf("i %d\n",i);
	      printf("followingmatrix\n");
	      printMatrix(followingmatrix);
	      printf("matrixmult\n");
	      printMatrix(matrixmult);
	    }
	  
	  negativeMatrix(matrixmult,matrixmult);
	  
	  if (debug & 12)
	    {
	      printf("matrixmult\n");
	      printMatrix(matrixmult);
	    }
	  
	  window = 
	    windowMatrix(companionMatrix,0,
			 i*matsize,bigsize-matsize,
			 matsize,matsize);

	  if (debug & 12)
	    {
	      printf("window\n");
	      printMatrix(window);
	    }

	  copyMatrix(matrixmult,window);

	  freeMatrix(matrixmult);
	  freeWindowMatrix(window);
	} /* for (i = 0; i < maxdeg; ++i) */

      freeMatrix(invleadingmatrix);

      condcompanionMatrix = Condition_Number_(companionMatrix);
      
      if (condcompanionMatrix > 1e12)
	{
	  if (debug & 12)
	    {
	      printf("identity transformation didnt succeed\n");
	      printf("companion matrix condition number %f too high\n",
		     condcompanionMatrix);
	    }
	  *process = PROCESS_UNKNOWN;
	}

    } /* if (condleadingmatrix < 1e6 && *process == PROCESS_UNKNOWN)
       */

  if (*process == PROCESS_IDENTITY)
    return;

  if (debug & 12)
    printf("identity transformation did not succeed\n");

  trailingmatrix = matPoly->matPoly[0];
      
  condtrailingmatrix = Condition_Number_(trailingmatrix);
      
  if (debug & 12)
    {
      printf("trailing matrix\n");
      printMatrix(trailingmatrix);
      printf("condtrailingmatrix %f\n",condtrailingmatrix);
    }
  
  if (condtrailingmatrix < 1e6 && !(debug & 0x200))
    {
      
      if (debug & 12)
	{
	  printf("condtrailingmatrix < 1e6\n");
	  printf("-> inverse process\n");
	}

      *process = PROCESS_INVERSE;
      
      invtrailingmatrix = inverseMatrix(trailingmatrix,0);
      
      if (debug & 12)
	{
	  printf("invtrailingmatrix\n");
	  printMatrix(invtrailingmatrix);
	}
      
      for (i = 0; i < maxdeg; ++i)
	{
	  int j;
	  
	  j = maxdeg - i;
	  
	  followingmatrix = matPoly->matPoly[j];
	  
	  matrixmult =
	    multiplyMatrix(invtrailingmatrix,followingmatrix,0);
	  
	  if (debug & 12)
	    {
	      printf("i %d j %d\n",i,j);
	      printf("followingmatrix\n");
	      printMatrix(followingmatrix);
	      printf("matrixmult\n");
	      printMatrix(matrixmult);
	    }
	  
	  negativeMatrix(matrixmult,matrixmult);
	  
	  if (debug & 12)
	    {
	      printf("matrixmult\n");
	      printMatrix(matrixmult);
	    }
	  
	  window = 
	    windowMatrix(companionMatrix,0,
			 i*matsize,bigsize-matsize,
			 matsize,matsize);
	  
	  if (debug & 12)
	    {
	      printf("window\n");
	      printMatrix(window);
	    }
	  
	  copyMatrix(matrixmult,window);
	  
	  freeMatrix(matrixmult);
	  freeWindowMatrix(window);
	} /* for (i = 0; i < maxdeg; ++i) */
	
      
      freeMatrix(invtrailingmatrix);
      
      condcompanionMatrix = Condition_Number_(companionMatrix);
	  
      if (condcompanionMatrix > 1e12)
	{
	  if (debug & 12)
	    {
	      printf("inverse transformation didnt succeed\n");
	      printf("companion matrix condition number %f too high\n",
		     condcompanionMatrix);
	    }
	  *process = PROCESS_UNKNOWN;
	}
    } /* if (condleadingmatrix < 1e6 && *process == PROCESS_UNKNOWN)
       */

  if (*process == PROCESS_INVERSE)
    return;

  if (debug & 12)
    {
      printf("inverse transformation did not succeed\n");
      printf("-> try random (as+b)/(cs+d) transformations\n");
    }

  smallestcond = 1e40;
  bestalpha = 0;
  bestbeta  = 0;
  bestgamma = 0;
  bestdelta = 0;

  /* try alot (40) of different random (alpha,beta,gamma,delta) 
     combinations for the purpose of using the combination which had
     the lowest condition number
  */

  for (i = 0; i < 40; ++i)
    {

      /* generate random (alpha,beta,gamma,delta) combination */

      alpha = drand48();
      beta  = drand48();
      gamma = drand48();
      delta = drand48();

      if (debug & 12)
	{
	  printf("alpha %f beta %f gamma %f delta %f\n",
		 alpha,beta,gamma,delta);
	}

      /* compute the leading matrix s^maxdeg where t=(as+b)/(cs+d) the
	 leading matrix is computed by adding together the matrix
	 monomials variable^0, variable^1, variable^2, ...,
	 variable^maxdeg, all weighted according to the s^maxdeg
	 coefficient of the rational transformation (as + b)^degree *
	 (cs + d)^(maxdeg-degree) for degree={0,1,2,...,maxdeg}
      */

      leadingmatrix = zerosMatrix(matsize,matsize);

      for (j = 0; j <= maxdeg; ++j)
	{
	  rationalfactor = pow(alpha,j)*pow(gamma,maxdeg-j);
	  scalematrix =
	    scaleMatrix(matPoly->matPoly[j],rationalfactor,0);
	  addMatrix(leadingmatrix,scalematrix,leadingmatrix);
	  freeMatrix(scalematrix), scalematrix = NULL;
	}
      
      condleadingmatrix = Condition_Number_(leadingmatrix);

      if (debug & 12)
	{
	  printf("leading matrix\n");
	  printMatrix(leadingmatrix);
	  printf("condleadingmatrix %f\n",condleadingmatrix);
	}

      freeMatrix(leadingmatrix), leadingmatrix = NULL;

      /* check if this is the best combination so far (i.e., it has
	 the lowest condition number), and if so, save this
	 combination
      */

    if (condleadingmatrix < smallestcond)
      {
	smallestcond = condleadingmatrix;
	bestalpha = alpha;
	bestbeta  = beta;
	bestgamma = gamma;
	bestdelta = delta;
      }
    }
  
  if (debug & 12)
    {
      printf("bestalpha %f bestbeta %f bestgamma %f bestdelta %f\n",
	     bestalpha,bestbeta,bestgamma,bestdelta);
    }

  if (smallestcond < 1e8)
    {

      if (debug & 12)
	{
	  printf("smallestcond < 1e8\n");
	  printf("-> rational_quotient process\n");
	}

      *process = PROCESS_RATIONAL_QUOTIENT;

      alpha = procvals[0] = bestalpha;
      beta  = procvals[1] = bestbeta;
      gamma = procvals[2] = bestgamma;
      delta = procvals[3] = bestdelta;

      /* recompute the leading matrix s^maxdeg where t=(as+b)/(cs+d)
	 using the best combination of random coefficients the
	 leading matrix is computed by adding together the matrix
	 monomials variable^0, variable^1, variable^2, ...,
	 variable^maxdeg, all weighted according to the s^maxdeg
	 coefficient of the rational transformation (as +
	 b)^degree * (cs + d)^(maxdeg-degree) for
	 degree={0,1,2,...,maxdeg} 
      */

      leadingmatrix = zerosMatrix(matsize,matsize);

      for (j = 0; j <= maxdeg; ++j)
	{
	  rationalfactor = pow(alpha,j)*pow(gamma,maxdeg-j);
	  scalematrix =
	    scaleMatrix(matPoly->matPoly[j],rationalfactor,0);
	  addMatrix(leadingmatrix,scalematrix,leadingmatrix);
	  freeMatrix(scalematrix), scalematrix = NULL;
	}

      if (debug & 12)
	{
	  printf("leading matrix\n");
	  printMatrix(leadingmatrix);
	}

      /* Compute the inverse of the leading matrix */

      invleadingmatrix = inverseMatrix(leadingmatrix,0);

      if (debug & 12)
	{
	  printf("inverseleading matrix\n");
	  printMatrix(invleadingmatrix);
	}

      for (i = 0; i < maxdeg; ++i)
	{
	  /* Compute the coefficient matrix for s^i this matrix is
	     computed by adding together the matrix monomials
	     variable^0, variable^1, variable^2, ...,
	     variable^maxdeg, all weighted according to the s^i
	     coefficient of the rational transformation (as +
	     b)^degree * (cs + d)^(maxdeg-degree) for
	     degree={0,1,2,...,maxdeg}
	  */

	  followingmatrix = zerosMatrix(matsize,matsize);

	  for (j = 0; j <= maxdeg; ++j)
	    {
	      rationalfactor =
		rationaltrans(alpha,beta,gamma,delta,maxdeg,i,j);
	      scalematrix =
		scaleMatrix(matPoly->matPoly[j],rationalfactor,0);
	      if (debug & 12)
		{
		  printf("rationalfactor %f i %d j %d scalematrix\n",
			 rationalfactor,i,j);
		  printMatrix(scalematrix);
		}
	      addMatrix(followingmatrix,scalematrix,followingmatrix);
	      freeMatrix(scalematrix), scalematrix = NULL;
	    }

	  if (debug & 12)
	    {
	      printf("following matrix\n");
	      printMatrix(followingmatrix);
	    }

	  matrixmult =
	    multiplyMatrix(invleadingmatrix,followingmatrix,0);
	  
	  negativeMatrix(matrixmult,matrixmult);

	  if (debug & 12)
	    {
	      printf("matrixmult\n");
	      printMatrix(matrixmult);
	    }
      
	  window = 
	    windowMatrix(companionMatrix,0,
			 i*matsize,bigsize-matsize,
			 matsize,matsize);
	  
	  if (debug & 12)
	    {
	      printf("window\n");
	      printMatrix(window);
	    }
	  
	  copyMatrix(matrixmult,window);
	  freeMatrix(followingmatrix);
	  freeMatrix(matrixmult);
	  freeWindowMatrix(window);
	} /* for (i = 0; i < maxdeg; ++i) */

      freeMatrix(leadingmatrix);
      freeMatrix(invleadingmatrix);
    }
}
   
void testCompanionMatrixLinear1()
{ 
  matrixPolynomial *matPoly;
  int i,j,k;
  matrix *compMat;
  int process;
  double procvals[4];

  double matPolyCoeffs[2][5][5] = 
  {
    {{ 2, 0, 3, 4, 0},
     { 5, 6, 0, 7, 0},
     { 0, 8, 9, 8, 6},
     { 3, 2, 0, 1, 4},
     { 0, 5, 4, 3, 2}},
    {{ 1, 0, 0, 0, 0},
     { 0, 1, 0, 0, 0},
     { 0, 0, 0, 0, 0},
     { 0, 0, 0, 0, 0},
     { 0, 0, 0, 0, 0}}
  };
  
  matPoly = zerosMatrixPolynomial(1/*degree*/,5/*size*/);
  printMatrixPolynomial(matPoly);

  for (i = 0; i < 2; ++i)
    for (j = 0; j < 5; ++j)
      for (k = 0; k < 5; ++k)
	matPoly->matPoly[i]->mat[j][k] = matPolyCoeffs[i][j][k];
  
  printMatrixPolynomial(matPoly);

  compMat = zerosMatrix(1,1);

  companionMatrixLinear(matPoly,compMat,&process,procvals,0);
  
  printMatrix(compMat);
  printf("process %d procvals %f %f %f %f\n",
	 process,procvals[0],procvals[1],procvals[2],procvals[3]);
  
  freeMatrixPolynomial(matPoly);
  freeMatrix(compMat);
}

void testCompanionMatrixLinear2()
{ 
  matrixPolynomial *matPoly;
  int i,j,k;
  matrix *compMat;
  int process;
  double procvals[4];
  cmatrix *VALS;
  cmatrix *VECTS;

  double matPolyCoeffs[2][10][10] = 
  {
    {{ 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
     { 0,-4, 0, 6, 3, 0, 1, 0, 0, 0,},
     { 0, 0,-4, 0, 6, 3, 0, 1, 0, 0,},
     { 0, 5, 0, 2,-7, 0, 0, 0, 1, 0,},
     { 0, 0, 5, 0, 2,-7, 0, 0, 0, 1,}},
    {{-1, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0,-1, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0,-1, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0,-1, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0,-1, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0,-1, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,}}
  };
  
  matPoly = zerosMatrixPolynomial(1/*degree*/,10/*size*/);
  printMatrixPolynomial(matPoly);

  for (i = 0; i < 2; ++i)
    for (j = 0; j < 10; ++j)
      for (k = 0; k < 10; ++k)
	matPoly->matPoly[i]->mat[j][k] = -matPolyCoeffs[i][j][k];
  
  printMatrixPolynomial(matPoly);

  compMat = zerosMatrix(1,1);

  companionMatrixLinear(matPoly,compMat,&process,procvals,0);
  
  printMatrix(compMat);
  printf("process %d procvals %f %f %f %f\n",
	 process,procvals[0],procvals[1],procvals[2],procvals[3]);

  VALS = zerosCMatrix(1,1);
  VECTS = zerosCMatrix(1,1);

  eig(compMat,VALS,VECTS);
  printCMatrix(VALS);
  printCMatrix(VECTS);
  
  freeMatrixPolynomial(matPoly);
  freeMatrix(compMat);
  freeCMatrix(VALS);
  freeCMatrix(VECTS);
}

void testCompanionMatrix1()
{ 
  matrixPolynomial *matPoly;
  int i,j,k;
  matrix *compMat;
  int process;
  double procvals[4];
  cmatrix *VALS;
  cmatrix *VECTS;

  double matPolyCoeffs[2][10][10] = 
  {
    {{ 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
     { 0,-4, 0, 6, 3, 0, 1, 0, 0, 0,},
     { 0, 0,-4, 0, 6, 3, 0, 1, 0, 0,},
     { 0, 5, 0, 2,-7, 0, 0, 0, 1, 0,},
     { 0, 0, 5, 0, 2,-7, 0, 0, 0, 1,}},
    {{-1, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0,-1, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0,-1, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0,-1, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0,-1, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0,-1, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,}}
  };
  
  matPoly = zerosMatrixPolynomial(1/*degree*/,10/*size*/);
  printMatrixPolynomial(matPoly);

  for (i = 0; i < 2; ++i)
    for (j = 0; j < 10; ++j)
      for (k = 0; k < 10; ++k)
	matPoly->matPoly[i]->mat[j][k] = -matPolyCoeffs[i][j][k];
  
  printMatrixPolynomial(matPoly);

  compMat = zerosMatrix(1,1);

  companionMatrix(matPoly,compMat,&process,procvals,0);
  
  printMatrix(compMat);
  printf("process %d procvals %f %f %f %f\n",
	 process,procvals[0],procvals[1],procvals[2],procvals[3]);

  VALS = zerosCMatrix(1,1);
  VECTS = zerosCMatrix(1,1);

  eig(compMat,VALS,VECTS);
  printCMatrix(VALS);
  printCMatrix(VECTS);

  freeMatrixPolynomial(matPoly);
  freeMatrix(compMat);
  freeCMatrix(VALS);
  freeCMatrix(VECTS);
}

void testCompanionMatrix2()
{ 
  matrixPolynomial *matPoly;
  int i,j,k;
  matrix *compMat;
  int process;
  double procvals[4];
  cmatrix *VALS;
  cmatrix *VECTS;

  double matPolyCoeffs[3][2][2] = 
  {
    {{-38, -5,},
     { -5, -2,}},
    {{ 48,  7,},
     {  7, -0,}},
    {{ -6, -1,},
     { -1, -0,}}
  };

  matPoly = zerosMatrixPolynomial(2/*degree*/,2/*size*/);
  printMatrixPolynomial(matPoly);

  for (i = 0; i < 3; ++i)
    for (j = 0; j < 2; ++j)
      for (k = 0; k < 2; ++k)
	matPoly->matPoly[i]->mat[j][k] = matPolyCoeffs[i][j][k];
  
  printMatrixPolynomial(matPoly);

  compMat = zerosMatrix(1,1);

  companionMatrix(matPoly,compMat,&process,procvals,0xffff);
  
  printMatrix(compMat);
  printf("process %d procvals %f %f %f %f\n",
	 process,procvals[0],procvals[1],procvals[2],procvals[3]);

  VALS = zerosCMatrix(1,1);
  VECTS = zerosCMatrix(1,1);

  eig(compMat,VALS,VECTS);
  printCMatrix(VALS);
  printCMatrix(VECTS);

  freeMatrixPolynomial(matPoly);
  freeMatrix(compMat);
  freeCMatrix(VALS);
  freeCMatrix(VECTS);
}

int testCompanionMatrix()
{
  testCompanionMatrixLinear1();
  testCompanionMatrixLinear2();
  testCompanionMatrix1();
  testCompanionMatrix2();
}
