/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#ifndef compmat_h
#define compmat_h

#include "matrix.h"
#include "matpoly.h"

enum process 
{
  PROCESS_IDENTITY = 0,		  /* don't use any transformations */
  PROCESS_INVERSE = 1,		  /* use t = 1/s transformation    */
  PROCESS_RATIONAL_QUOTIENT = 2,  /* use t = (as+b)/(cs+d)
				     transformation */ 
  PROCESS_UNKNOWN = -1		  /* transformation not known yet  */
};

#ifdef GCC_PROTOTYPES

int isLinearCompanionMatrix(matrixPolynomial *matPoly);
/* returns	boolean characterizing whether or not the matrix
   		polynomial is linear (order 1)
*/

void companionMatrixLinear
(matrixPolynomial *matPoly,
 matrix *companionMatrix,
 int *process,
 double *procvals,
 int debug);
/* effect	Computes the companion matrix for a linear matrix
   		satisfying the property that the upperlefthand block
		of the linear term is an identity matrix. Uses the
		simplification that the companion matrix 
		M' == M11 - M12 * M22^{-1} *M21 where M has been
		subdivided according to the upperlefthand block of the
		linear term
		[ M11 + I | M12 ]
		[ M21     | M22 ]

*/

int nChooseK(int n,int k);
/* returns	the combinatoric value of n choose k
 */

double rationaltrans(double alpha, double beta, double gamma, double delta,
		     int maxdeg, int i, int j);
/* returns	The s^spower coefficient of t^tpower assuming t=(as+b)/(cs+d)
   requires	maxtpower is an integer > 0
		tpower is an integer >= 0
		spower is an integer >= 0
*/

void companionMatrix
(matrixPolynomial *matPoly,
 matrix *companionMatrix,
 int *process,
 double *procvals,
 int debug);
/* effect	Compute the companion matrix corresponding to the
   		given matrix polynomial. Use the condition numbers to
		choose the appropriate transformation: identity,
		inverse, or rational quotient
*/

#else
void companionMatrix();
#endif

#endif /* compmat_h */
