/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#include <math.h>
#include <assert.h>
#include "linutil.h"
#include "complex.h"
#include "matrix.h"

double Condition_Number_(M)
matrix *M;
{
  int n;
  unsigned char JOBU='N';
  unsigned char JOBVT='N';
  complex *A;
  double *S;
  int LWORK;
  complex *WORK; 
  double *RWORK; 
  int INFO;
  double minEIG, maxEIG;
  int i;
  int one=1;

  assert(M->width == M->height);

  n = M->width;
  A = createComplexMatrix(M);
  S = (double *)malloc(n*sizeof(double));
  LWORK = 10*n*n;
  WORK = (complex *)malloc(LWORK*sizeof(complex));
  RWORK = (double *)malloc(LWORK*sizeof(double));

  zgesvd_(&JOBU,&JOBVT,&n,&n,A,&n,S,NULL,&one,NULL,&one,WORK,&LWORK,RWORK,&INFO);

  minEIG = maxEIG = fabs(S[0]);
  for (i = 1; i < n; i++){
    if (fabs(S[i]) < minEIG) minEIG = fabs(S[i]);
    if (fabs(S[i]) > maxEIG) maxEIG = fabs(S[i]);
  }
  free(A);
  free(S);
  free(WORK);
  free(RWORK);
  return(minEIG/maxEIG);
}

int testCond1()
{
  matrix *id3;
  double cond3;
  id3 = eyeMatrix(3);
  cond3 = Condition_Number_(id3);
  printf("%f\n",cond3);
  freeMatrix(id3);
}

int testCond2()
{
  matrix *lin3;
  double cond3;
  lin3 = makeLinearMatrix(3);
  cond3 = Condition_Number_(lin3);
  printf("%f\n",cond3);
  freeMatrix(lin3);
}

int testCond()
{
  testCond1();
  testCond2();
}
