/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#include "complex.h"
#include "cmatrix.h"
#include "linutil.h"
#include <assert.h>

void eig(M,VALS,VECTS)
matrix *M;
cmatrix *VALS;
cmatrix *VECTS;
{
  int n;
  unsigned char JOBVL='V';
  unsigned char JOBVR='N';
  complex *A;
  complex *W;
  complex *VR;
  int LWORK;
  complex *WORK; 
  double *RWORK; 
  int INFO;
  int one=1;
  int i;
  int nsquared;

  assert(M->width == M->height);

  n = M->width;
  nsquared = n*n;

  A = createComplexMatrix(M);
  W = (complex *)malloc(n*sizeof(complex));
  memset(W,0,n*sizeof(complex));
  VR = (complex *)malloc(n*n*sizeof(complex));
  memset(VR,0,n*n*sizeof(complex));
  LWORK = 10*n*n;
  WORK = (complex *)malloc(LWORK*sizeof(complex));
  RWORK = (double *)malloc(LWORK*sizeof(double));

  zgeev_(&JOBVL,&JOBVR,&n,A,&n,W,VR,&n,NULL,&one,WORK,&LWORK,RWORK,&INFO);

  resizeCMatrix(VALS,1,n);

  for (i = 0; i < n; ++i)
    VALS->mat[i][0] = W[i];
  
  resizeCMatrix(VECTS,n,n);
  
  convertComplexArray(VR,VECTS->mat,n,n);

  free(A);
  free(W);
  free(VR);
  free(WORK);
  free(RWORK);
}

int testEig0()
{
  double coeffs[2][2] =
  { {1,2},
    {3,4}
  };
  int i,j;
  matrix *M;
  cmatrix *VALS;
  cmatrix *VECTS;

  VALS = zerosCMatrix(1,1);
  VECTS = zerosCMatrix(1,1);
  
  M = zerosMatrix(2,2);
  for (i = 0; i < 2; ++i)
    for (j = 0; j < 2; ++j)
      M->mat[i][j] = coeffs[i][j];
  eig(M,VALS,VECTS);
  printCMatrix(VALS);
  printCMatrix(VECTS);
}

int testEig1()
{
  double coeffs[3][3] =
  { {1,3,7},
    {9,8,2},
    {0,5,2}
  };
  int i,j;
  matrix *M;
  cmatrix *VALS;
  cmatrix *VECTS;

  VALS = zerosCMatrix(1,1);
  VECTS = zerosCMatrix(1,1);
  
  M = zerosMatrix(3,3);
  for (i = 0; i < 3; ++i)
    for (j = 0; j < 3; ++j)
      M->mat[i][j] = coeffs[i][j];
  printMatrix(M);
  eig(M,VALS,VECTS);
  printCMatrix(VALS);
  printCMatrix(VECTS);
}

int testEig2()
{
  double coeffs[3][3] =
  { {1,0,0},
    {0,2,0},
    {0,0,7}
  };
  int i,j;
  matrix *M;
  cmatrix *VALS;
  cmatrix *VECTS;

  VALS = zerosCMatrix(1,1);
  VECTS = zerosCMatrix(1,1);
  
  M = zerosMatrix(3,3);
  for (i = 0; i < 3; ++i)
    for (j = 0; j < 3; ++j)
      M->mat[i][j] = coeffs[i][j];
  eig(M,VALS,VECTS);
  printCMatrix(VALS);
  printCMatrix(VECTS);
}

int testEig3()
{
  double coeffs[6][6] =
  { {0, 0,-1, 0, 0, 0,},
    {0, 0, 0, 0,-1, 0,},
    {0, 0, 0, 0, 0,-1,},
    {0, 0,-4, 0, 6, 3,},
    {0, 5, 0, 2,-7, 0,},
    {0, 0, 5, 0, 2,-7,}
  };
  int i,j;
  matrix *M;
  cmatrix *VALS;
  cmatrix *VECTS;

  VALS = zerosCMatrix(1,1);
  VECTS = zerosCMatrix(1,1);
  
  M = zerosMatrix(6,6);
  for (i = 0; i < 6; ++i)
    for (j = 0; j < 6; ++j)
      M->mat[i][j] = coeffs[i][j];
  eig(M,VALS,VECTS);
  printCMatrix(VALS);
  printCMatrix(VECTS);
}


int testEig()
{
  testEig0();
  testEig1();
  testEig2();
  testEig3();
}
  
