/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#include "linutil.h"
#include <math.h>
#include <assert.h>

void invertArray(SRC,INV,size)
double **SRC,**INV;
int size;
{
  double *A;
  int *IPIV;
  int LWORK;
  double *WORK; 
  int INFO;

  assert(INV);

  A = createArray(SRC,size,size);
  IPIV = (int *)malloc((size+1)*sizeof(int));
  LWORK = size*size;
  WORK = (double *)malloc(LWORK*sizeof(double));
  dgetrf_(&size,&size,A,&size,IPIV,&INFO);
  dgetri_(&size,A,&size,IPIV,WORK,&LWORK,&INFO);

  convertArray(A,INV,size,size);
}

int testInv1()
{
  double **id3,**invid3;
  id3 = eyeArray(3);
  invid3 = eyeArray(3);
  printArray(invid3,3,3);
  invertArray(id3,invid3,3);
  printArray(invid3,3,3);
  freeArray(id3);
  freeArray(invid3);
}

int testInv2()
{
  double **id3,**invid3,**mul3;
  id3 = makeLinearArray(3);
  invid3 = makeLinearArray(3);
  printArray(invid3,3,3);
  invertArray(id3,invid3,3);
  printArray(invid3,3,3);
  mul3 = makeLinearArray(3);
  multiplyArray(id3,invid3,mul3,3,3,3);
  printArray(mul3,3,3);
  freeArray(id3);
  freeArray(invid3);
}

int testInv3()
{
  double coeffs[3][3] =
  { {1,3,7},
    {9,8,2},
    {0,5,2}
  };
  int i,j;

  double **id3,**invid3,**mul3;
  id3 = makeLinearArray(3);
  for (i = 0; i < 3; ++i)
    for (j = 0; j < 3; ++j)
      id3[i][j] = coeffs[i][j];

  invid3 = makeLinearArray(3);
  printArray(invid3,3,3);
  invertArray(id3,invid3,3);
  printArray(invid3,3,3);
  mul3 = makeLinearArray(3);
  multiplyArray(id3,invid3,mul3,3,3,3);
  printArray(mul3,3,3);
  freeArray(id3);
  freeArray(invid3);
}

int testInv()
{
  testInv1();
  testInv2();
  testInv3();
}
