/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#include <math.h>
#include "complex.h"

double *createArray(M,w,h)
double **M;
int w,h;
{
  double *A = (double *)malloc(w*h*sizeof(double));
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      A[i*w+j] = M[i][j];
  return A;
}

void printArray(M,w,h)
double **M;
int w;
int h;
{
  int i,j;
  
  for (i = 0; i < h; ++i)
    {
      printf(" [ ");

      for (j = 0; j < w; ++j)
	printf("%12.6f ",M[i][j]);
      
      printf(" ]\n");
    }
}

void convertArray(L,M,w,h)
double *L,**M;
int w,h;
{
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      M[i][j] = L[i*w+j];
}

complex *createComplexArray(M,w,h)
double **M;
int w,h;
{
  complex *A = (complex *)malloc(w*h*sizeof(complex));
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      {
	A[i*w+j].real = M[i][j];
	A[i*w+j].imag = 0;
      }
  return A;
}

double **eyeArray(n)
int n;
{
  double **ans;
  int i;
  ans = (double **)malloc(n*sizeof(double *));

  for (i = 0; i < n ; ++i)
    {
      ans[i] = (double *)malloc(n*sizeof(double));
      memset(ans[i],0,n*sizeof(double));
      ans[i][i] = 1;
    }
  
  return ans;
}

double **zerosArray(w,h)
int w,h;
{
  double **ans;
  int i;
  ans = (double **)malloc(h*sizeof(double *));

  for (i = 0; i < h ; ++i)
    {
      ans[i] = (double *)malloc(w*sizeof(double));
      memset(ans[i],0,w*sizeof(double));
    }
  
  return ans;
}

double **makeLinearArray(n)
int n;
{
  double **ans;
  int i;
  ans = (double **)malloc(n*sizeof(double *));

  for (i = 0; i < n ; ++i)
    {
      ans[i] = (double *)malloc(n*sizeof(double));
      memset(ans[i],0,n*sizeof(double));
      ans[i][i] = i+1;
    }
  
  return ans;
}

void freeArray(M,h)
double **M;
int h;
{
  int i;

  if (!M) return;

  for (i = 0; i < h; ++i)
    free(M[i]), M[i] = NULL;
  free(M);
}

void freeWindowArray(M)
double **M;
{
  int i;

  if (!M) return;

  free(M);
}

void addArray(M1,M2,DST,w,h)
double **M1,**M2,**DST;
int w,h;
{
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      DST[i][j] = M1[i][j] + M2[i][j];
}

void subtractArray(M1,M2,DST,w,h)
double **M1,**M2,**DST;
int w,h;
{
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      DST[i][j] = M1[i][j] - M2[i][j];
}

void multiplyArray(M1,M2,DST,w,h,w2)
double **M1,**M2,**DST;
int w,h;
{
  int i,j,k;
  double sum;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w2; ++j)
      {
	sum = 0;
	for (k = 0; k < w; ++k)
	  sum += M1[i][k] * M2[k][j];
	DST[i][j] = sum;
      }
}

void copyArray(SRC,DST,w,h)
double **SRC,**DST;
int w,h;
{
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      DST[i][j] = SRC[i][j];
}

void subArray(SRC,DST,x,y,w,h)
double **SRC,**DST;
int x,y,w,h;
{
  int i,j;

  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      DST[i][j] = SRC[i+y][j+x];
}

double mypower(val, exponent)
double val, exponent;
{
  if (val < 0)
    return -pow(-val,exponent);
  return pow(val,exponent);
}

void windowArray(SRC,DST,x,y,w,h)
double **SRC,**DST;
int x,y,w,h;
{
  int i,j;

  for (i = 0; i < h; ++i)
    DST[i] = &SRC[i+y][x];
}

void negativeArray(M1,DST,w,h)
double **M1,**DST;
int w,h;
{
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      DST[i][j] = -M1[i][j];
}

void scaleArray(M1,s,DST,w,h)
double **M1,s,**DST;
int w,h;
{
  int i,j;
  for (i = 0; i < h; ++i)
    for (j = 0; j < w; ++j)
      DST[i][j] = s*M1[i][j];
}

