/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#ifndef matpoly_h
#define matpoly_h

#include "matrix.h"

/* matrixPolynomial class characterizes matrix polynomials of double
   precision values.  The "matPoly" member is an array of pointers to
   matrix objects.  and matPoly[i] points to the array of degree
   i. The matrixPolynomial object also knows about the size of the
   square matrix as well as its maximum degree
*/

typedef struct matrixPolynomial
{
  matrix **matPoly;  /* array of pointers to matrices */
  int degree;        /* degree of matrix polynomial   */
  int size;          /* size of square matrices       */
} matrixPolynomial;

#ifdef GCC_PROTOTYPES

void printMatrixPolynomial(matrixPolynomial *MatPoly);
/* effect	prints the matrix polynomial to stdout
 */

matrixPolynomial *zerosMatrixPolynomial(int degree, int size);
/* returns	a newly allocated matrix polynomial including degree+1
   		squared matrices of given size 
*/

void freeMatrixPolynomial(matrixPolynomial *matPoly);
/* effect	deallocates memory associated with the given matrix polynomial
 */
#else

matrixPolynomial *zerosMatrixPolynomial();
void freeMatrixPolynomial();

#endif
#endif /* matpoly_h */
