/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#include "matrix.h"
#include "linutil.h"
#include <assert.h>

void printMatrix(mat)
matrix *mat;
{
  printArray(mat->mat,mat->width,mat->height);
}

matrix *zerosMatrix(w,h)
int w,h;
{
  matrix *ans;
  ans = (matrix *)malloc(sizeof(matrix));
  memset(ans,0,sizeof(matrix));
  
  ans->mat = zerosArray(w,h);
  ans->width = w;
  ans->height = h;
  
  return ans;
}

matrix *eyeMatrix(n)
int n;
{
  matrix *ans;
  ans = (matrix *)malloc(sizeof(matrix));
  memset(ans,0,sizeof(matrix));
  
  ans->mat = eyeArray(n);
  ans->width = n;
  ans->height = n;
  return ans;
}

matrix *makeLinearMatrix(n)
int n;
{
  matrix *ans;
  ans = (matrix *)malloc(sizeof(matrix));
  memset(ans,0,sizeof(matrix));
  
  ans->mat = makeLinearArray(n);
  ans->width = n;
  ans->height = n;
  return ans;
}

void freeMatrix(mat)
matrix *mat;
{
  freeArray(mat->mat,mat->height);
  mat->mat = NULL;
  mat->width = 0;
  mat->height = 0;
  free(mat);
}

matrix *addMatrix(m1,m2,dst)
matrix *m1;
matrix *m2;
matrix *dst;
{
  assert(m1->width == m2->width &&
	 m1->height == m2->height
	 );
  
  if (!dst)
    dst = zerosMatrix(m1->width,m1->height);
 
  assert(m1->width == dst->width &&
	 m1->height == dst->height
	 );

  addArray(m1->mat,m2->mat,dst->mat,m1->width,m1->height);
  
  return dst;
}

matrix *copyMatrix(src,dst)
matrix *src;
matrix *dst;
{
  if (!dst)
    dst = zerosMatrix(src->width,src->height);
 
  assert(src->width == dst->width &&
	 src->height == dst->height
	 );

  copyArray(src->mat,dst->mat,src->width,src->height);
  
  return dst;
}

matrix *subMatrix(src,dst,x,y,w,h)
matrix *src;
matrix *dst;
int x,y,w,h;
{
  if (!dst)
    dst = zerosMatrix(w,h);
 
  assert(dst->width == w &&
	 dst->height == h 
	 );

  subArray(src->mat,dst->mat,x,y,w,h);
  
  return dst;
}

matrix *subtractMatrix(m1,m2,dst)
matrix *m1;
matrix *m2;
matrix *dst;
{
  assert(m1->width == m2->width &&
	 m1->height == m2->height
	 );
  
  if (!dst)
    dst = zerosMatrix(m1->width,m1->height);
 
  assert(m1->width == dst->width &&
	 m1->height == dst->height
	 );

  subtractArray(m1->mat,m2->mat,dst->mat,m1->width,m1->height);

  return dst;
}

matrix *multiplyMatrix(m1,m2,dst)
matrix *m1;
matrix *m2;
matrix *dst;
{

  assert(m1->width == m2->height
	 );
  
  if (!dst)
    dst = zerosMatrix(m2->width,m1->height);
 
  assert(m2->width == dst->width &&
	 m1->height == dst->height
	 );

  multiplyArray(m1->mat,m2->mat,dst->mat,m1->width,m1->height,m2->width);

  return dst;
}

matrix *resizeMatrix(m,w,h)
matrix *m;
int w,h;
{
  if (m->width == w &&
      m->height == h)
    return;
 
  freeArray(m->mat,m->height);
  m->mat = zerosArray(w,h);
  m->width = w;
  m->height = h;

  return m;
}
 
matrix *inverseMatrix(m,dst)
matrix *m,*dst;
{  
  assert(m->width == m->height);

  if (!dst)
    dst = zerosMatrix(m->width,m->height);
 
  assert(m->width == dst->width &&
	 m->height == dst->height);

  invertArray(m->mat,dst->mat,m->width);

  return dst;
}

matrix *windowMatrix(src,dst,x,y,w,h)
matrix *src;
matrix *dst;
int x,y,w,h;
{
  if (!dst)
    {
      dst = (matrix *)malloc(sizeof(matrix));
      memset(dst,0,sizeof(matrix));
    }
  else
    freeArray(dst->mat,dst->height);

  dst->mat = (double **)malloc(h*sizeof(double *));
  dst->width = w;
  dst->height = h;
 
  windowArray(src->mat,dst->mat,x,y,w,h);
  
  return dst;
}

void freeWindowMatrix(mat)
matrix *mat;
{
  freeWindowArray(mat->mat);
  mat->mat = NULL;
  mat->width = 0;
  mat->height = 0;
  free(mat);
}

matrix *negativeMatrix(m1,dst)
matrix *m1;
matrix *dst;
{

  if (!dst)
    dst = zerosMatrix(m1->width,m1->height);

  assert(m1->width == dst->width &&
	 m1->height == dst->height
	 );
  
  negativeArray(m1->mat,dst->mat,m1->width,m1->height);

  return dst;
}

matrix *scaleMatrix(m1,s,dst)
matrix *m1;
double s;
matrix *dst;
{

  if (!dst)
    dst = zerosMatrix(m1->width,m1->height);

  assert(m1->width == dst->width &&
	 m1->height == dst->height
	 );
  
  scaleArray(m1->mat,s,dst->mat,m1->width,m1->height);

  return dst;
}
