/*
 * Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
 * All rights reserved. This material contains unpublished,
 * copyrighted work, which includes confidential and proprietary
 * information of Aaron Wallack.
 */

#include "resultant.h"
#include "compmat.h"
#include "cmatrix.h"
#include "linutil.h"
#include "testfns.h"
#include <assert.h>

void solveResultant(Resultant, solns,debug)
resultant *Resultant;
cmatrix *solns;
int debug;
{
  matrix *compMat;
  cmatrix *eigenVals;
  cmatrix *eigenVecs;
  int process;
  double procvals[4];
  int numHypothesizedSolutions;
  int numVariables;
  int i,j;
  int **extract;
  complex zeroComplex;
  zeroComplex.real = zeroComplex.imag = 0;

  compMat = zerosMatrix(1,1);

  companionMatrix(Resultant->matPoly,compMat,&process,procvals,debug);

  numHypothesizedSolutions = 
    Resultant->matPoly->size * Resultant->matPoly->degree;

  if (debug & 0x4)
    {
      printf("companionMatrix\n");
      printMatrix(compMat);
    }

  eigenVals = zerosCMatrix(1,1);
  eigenVecs = zerosCMatrix(1,1);
  
  eig(compMat,eigenVals,eigenVecs);

  if (debug & 0x4)
    {
      printf("eigenVals\n");
      printCMatrix(eigenVals);
      printf("eigenVecs\n");
      printCMatrix(eigenVecs);
    }

  numVariables = Resultant->numVars+1;
  
  if (Resultant->uDim)
    {
      numHypothesizedSolutions = Resultant->uDim;
      numVariables = Resultant->numVars;
    }

  if (debug & 0x4)
    {
      printf("numHypothesizedSolutions %d numVariables %d\n",
	     numHypothesizedSolutions, numVariables);
    }

  resizeCMatrix(solns,numVariables,numHypothesizedSolutions);

  extract = Resultant->extractionRecipe;
  
  for (i = 0; i < numHypothesizedSolutions; ++i)
    {
      if (debug & 0x4)
	printf("eigenval %f + %f i\n",
	       eigenVals->mat[i][0].real,
	       eigenVals->mat[i][0].imag);

      switch (process)
	{
	case PROCESS_IDENTITY:
	  solns->mat[i][0] = eigenVals->mat[i][0];
	  if (debug & 0x4)
	    printf("identity map\n");
	  break;
	case PROCESS_INVERSE:
	  solns->mat[i][0] = oneOverComplex(eigenVals->mat[i][0]);
	  if (debug & 0x4)
	    printf("inverse map\n");
	  break;
	case PROCESS_RATIONAL_QUOTIENT:
	  {
	    complex proc1,proc3,num,denom;
	    proc1.real = procvals[1];
	    proc1.imag = 0;
	    num = addComplex(proc1,
			     scaleComplex(eigenVals->mat[i][0],procvals[0]));
	    proc3.real = procvals[3];
	    proc3.imag = 0;
	    denom = addComplex(proc3,
			       scaleComplex(eigenVals->mat[i][0],procvals[2]));
	    
	    solns->mat[i][0] = divideComplex(num,denom);
	  }
	  if (debug & 0x4)
	    printf("rational quotient map\n");
	  break;
	default:
	  assert(0);
	}

      if (debug & 0x4)
	printf("becomes root %f + %f i\n",
	       solns->mat[i][0].real,
	       solns->mat[i][0].imag);
      
      for (j = 0; j < numVariables-1; ++j)
	{
	  if (Resultant->uDim)
	    {
	      if (!isZeroComplex(eigenVecs->mat[i][0]))
		{
		  solns->mat[i][j+1] = 
		    divideComplex
		    (mycpower(eigenVecs->mat[i][extract[j+1][0]-1],
			      1./extract[j+1][1]),
		     mycpower(eigenVecs->mat[i][extract[j+1][2]-1],
			      1./extract[j+1][3]));
		}
	      else
		{
		  solns->mat[i][j+1] = zeroComplex;
		}
	    }
	  else 
	    {
	      if (!extract[j][3])
		{
		  if (!isZeroComplex(eigenVecs->mat[i][extract[j][0]-1]))
		      solns->mat[i][j+1] = 
		      mycpower(eigenVecs->mat[i][extract[j][0]],
			      1./extract[j][1]);
		  else
		    solns->mat[i][j+1] = zeroComplex;
		}
	      else
		{
		  if (!isZeroComplex(eigenVecs->mat[i][extract[j][0]-1]) &&
		      !isZeroComplex(eigenVecs->mat[i][extract[j][2]-1]))
		    solns->mat[i][j+1] = 
		      divideComplex
		      (mycpower(eigenVecs->mat[i][extract[j][0]-1],
				1./extract[j][1]),
		       mycpower(eigenVecs->mat[i][extract[j][2]-1],
				1./extract[j][3]));
		  else
		    solns->mat[i][j+1] = zeroComplex;
		} /* if (!extract[j][3]) .. else */
	    } /* if (Resultant->uDim) .. else */
	} /* for (j = 0; j < numVariables; ++j) */

      if (debug & 0x4)
	{
	  printf("eigenvector\n");
	  for (j = 0; j < eigenVecs->width; ++j)
	    printf("%f + %f i\n",
		   eigenVecs->mat[i][j].real,
		   eigenVecs->mat[i][j].imag);
	  printf("\ncorresponds to root\n");
	  for (j = 0; j < numVariables; ++j)
	    printf("%f + %f i\n",
		   solns->mat[i][j].real,
		   solns->mat[i][j].imag);
	}
    } /* for (i = 0; i < numHypothesizedSolutions; ++i) */
}

void testSolveResultant1()
{ 
  int i,j,k;
  resultant res;
  cmatrix *solns;
  int extractionRecipe[3][4] =
  {{1,1,1,1},{2,1,1,1},{3,1,1,1,}};
  int *extractionRecipePtrs[3];

  double matPolyCoeffs[2][10][10] = 
  {
    {{ 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,},
     { 0,-4, 0, 6, 3, 0, 1, 0, 0, 0,},
     { 0, 0,-4, 0, 6, 3, 0, 1, 0, 0,},
     { 0, 5, 0, 2,-7, 0, 0, 0, 1, 0,},
     { 0, 0, 5, 0, 2,-7, 0, 0, 0, 1,}},
    {{-1, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0,-1, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0,-1, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0,-1, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0,-1, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0,-1, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,},
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,}}
  };
  
  for (i = 0; i < 3; ++i)
    extractionRecipePtrs[i] = &extractionRecipe[i][0];

  res.matPoly = zerosMatrixPolynomial(1/*degree*/,10/*size*/);

  for (i = 0; i < 2; ++i)
    for (j = 0; j < 10; ++j)
      for (k = 0; k < 10; ++k)
	res.matPoly->matPoly[i]->mat[j][k] = -matPolyCoeffs[i][j][k];
  
  /*printMatrixPolynomial(res.matPoly);*/

  res.uDim = 6;
  res.numVars = 3;
  res.extractionRecipe = extractionRecipePtrs;

  solns = zerosCMatrix(1,1);

  solveResultant(&res,solns,0xffff);

  printCMatrix(solns);

  freeMatrixPolynomial(res.matPoly);
  freeCMatrix(solns);
}

void testSolveResultant2()
{ 
  int i,j,k;
  resultant res;
  cmatrix *solns;
  int extractionRecipe[1][4] =
  {{2,1,1,1}};
  int *extractionRecipePtrs[1];

  double matPolyCoeffs[3][2][2] = 
  {
    {{-38, -5,},
     { -5, -2,}},
    {{ 48,  7,},
     {  7, -0,}},
    {{ -6, -1,},
     { -1, -0,}}
  };
  
  for (i = 0; i < 1; ++i)
    extractionRecipePtrs[i] = &extractionRecipe[i][0];

  res.matPoly = zerosMatrixPolynomial(2/*degree*/,2/*size*/);

  for (i = 0; i < 3; ++i)
    for (j = 0; j < 2; ++j)
      for (k = 0; k < 2; ++k)
	res.matPoly->matPoly[i]->mat[j][k] = -matPolyCoeffs[i][j][k];
  
  /*printMatrixPolynomial(res.matPoly);*/

  res.uDim = 0;
  res.numVars = 1;
  res.extractionRecipe = extractionRecipePtrs;

  solns = zerosCMatrix(1,1);

  solveResultant(&res,solns,0xffff);

  printCMatrix(solns);

  freeMatrixPolynomial(res.matPoly);
  freeCMatrix(solns);
}

void testSolveResultant3()
{ 
  int i,j,k;
  resultant res;
  cmatrix *solns;
  int extractionRecipe[1][4] =
  {{2,1,1,1}};
  int *extractionRecipePtrs[1];

  double matPolyCoeffs[3][2][2] = 
  {
    {{-38, -5,},
     { -5, -2,}},
    {{ 48,  7,},
     {  7, -0,}},
    {{ -6, -1,},
     { -1, -0,}}
  };
  double expectedR[4][2][2] =
  {
    {{ 7.115511, 0.807265}, { -2.585121, 2.918674}},
    {{ 7.115511,-0.807265}, { -2.585121,-2.918674}},
    {{ 0.888402, 0.0 }, {0.214778, 0.}},
    {{-1.119423, 0.0}, {-7.044536, 0.}}
  };
  
  complex expected[4][2];
  cmatrix *vals;
  cmatrix *rmnz;


  void ((*fn[2])(complex *, complex *)) =
  {
    fnX2_6X_3Y_4,
    fnY2_2X_7Y_5
  };

  for (i = 0; i < 1; ++i)
    extractionRecipePtrs[i] = &extractionRecipe[i][0];

  res.matPoly = zerosMatrixPolynomial(2/*degree*/,2/*size*/);

  for (i = 0; i < 3; ++i)
    for (j = 0; j < 2; ++j)
      for (k = 0; k < 2; ++k)
	res.matPoly->matPoly[i]->mat[j][k] = -matPolyCoeffs[i][j][k];
  
  res.uDim = 0;
  res.numVars = 1;
  res.extractionRecipe = extractionRecipePtrs;
  res.fn = fn;

  solns = zerosCMatrix(1,1);

  solveResultant(&res,solns);

  printCMatrix(solns);

  for (i = 0; i < 4; ++i)
    {
      for (j = 0; j < 2; ++j)
	{
	  expected[i][j].real = expectedR[i][j][0];
	  expected[i][j].imag = expectedR[i][j][1];
	}
      assert (containsCMatrix(solns,expected[i]));
    }

  vals = zerosCMatrix(1,1);

  for (i = 0; i < 4; ++i)
    {
      evalResultantAtX(fn,solns->mat[i],2,vals);
      printCMatrix(vals);
    }

  rmnz = removeNonZero(solns,&res,0);

  printCMatrix(rmnz);

  freeMatrixPolynomial(res.matPoly);
  freeCMatrix(solns);
}
