% Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
% All rights reserved. This material contains unpublished,
% copyrighted work, which includes confidential and proprietary
% information of Aaron Wallack.

% Bezier intersection example
% The following example implicitizes two Bezier curves and then computes
% their intersection. First, we compute a resultant in X,Y, and W which
% corresponds to the curve (xb1,yb1,wb1). dixres(X,Y,W) is a matrix polynomial
% in X,Y,W whose value vanishes for (x,y,w) points on the first curve.
% Then, we substitute xb2,yb2,wb2 into dixres(X,Y,W) producing a new matrix
% resultant in s alone. Then we numerically solve for s.

maple('read(`../maple/bezout.map`);');
maple('  xb1 := 4*s^3 + 9*s^2 + 7*s -11;');
maple('  yb1 :=13*s^3 - 5*s^2 + 2*s - 6;');
maple('  wb1 := 3*s^3 + 4*s^2 - 8*s    ;');

maple('  xb2 :=17*s^3 - 2*s^2 + 6*s + 9;');
maple('  yb2 :=   s^3 + 8*s^2 +   s + 5;');
maple('  wb2 :=-4*s^3 + 5*s^2 - 7*s + 1;');

maple('  f1 := X*wb1-W*xb1;');
maple('  f2 := Y*wb1-W*yb1;');

maple('  fnArray := array(1..2,[f1,f2]);');
maple('  varArray := array(1..1,[s]);');
maple('  dixres := Bezout(eval(fnArray),eval(varArray),s);');
maple('  dixres[5] := eval(map(proc(var) simplify(var/W) end, dixres[5]));');
maple('  dixres := resultantImplicitizeVariables(dixres,array(1..3,[X,Y,W]));');
maple('  diximp := resultantImplicitizeFunction(eval(dixres),array(1..3,[xb2,yb2,wb2]));');

% Convert the maple resultant into a matlab resultant
g=mapleresultant('diximp');

% Numerically solve the matlab resultant
x=solveresultant(g)


