% Copyright (c) 1998 by Aaron Wallack, Natick, MA USA
% All rights reserved. This material contains unpublished,
% copyrighted work, which includes confidential and proprietary
% information of Aaron Wallack.

% Example for computing intersection of Discretization Boundary Curves
% Discretization Boundary Curves were defined in a paper on scanning beam 
% object recognition. Basically, a DBC is a curve in configuration space
% wherein the discretized scanline value is at a discretization boundary.
% Note that we use Maple to use symbolic manipulation (subs())
% Finally, we arrive at a system of 2 equations (f1,f2) and construct
% a generic resultant, which can be used to solve for the intersection
% between a CBC and a DBC
% Note that we instantiate the values of the generic resultant before
% calling the numeric solver

% Set up equations

maple('read(`../maple/bezout.map`);');
maple('CBC := V_gamma * sin(theta + gamma) - S - y;');
maple('CBC_theta := subs(sin(theta+gamma)=sin(theta)*cos(gamma)+sin(gamma)*cos(theta),CBC);');
maple('CBC_t := subs({sin(theta)=2*t/(1+t*t),cos(theta)=(1-t*t)/(1+t*t)},CBC_theta);');
maple('I_alpha_theta := (R_alpha - (D_alpha - y)*sin(theta + alpha))/(cos(theta + alpha));');
maple('I_alpha_theta_p := subs({sin(theta+alpha)=sin(theta)*cos(alpha)+sin(gamma)*cos(alpha),cos(theta+alpha)=cos(theta)*cos(alpha)-sin(theta)*sin(alpha)},I_alpha_theta);');
maple('I_alpha_t := subs({sin(theta)=2*t/(1+t*t),cos(theta)=(1-t*t)/(1+t*t)},I_alpha_theta_p);');

% Formulate a set of two equations in two variables (and some other to-be
% instantiated variables)

maple('f1 := expand(simplify((numer(CBC_t)*denom(I_alpha_t))/(1+t*t)));');
maple('f2 := expand(simplify((denom(CBC_t)*numer(I_alpha_t))/((1+t*t)^2)));');
maple('fnarray:=array(1..2,[eval(f1),eval(f2)]);');
maple('vararray:=array(1..1,[t]);');

% Construct the genericitized resultant

maple('dix:=Bezout(eval(fnarray),eval(vararray),y);');

% Instantiate hte genericitized resultant

maple('dix2 := map(proc(x) expand(subs({V_gamma=1.,sin(gamma)=0.2,cos(gamma)=0.3,sin(alpha)=-0.1,cos(alpha)=0.4,R_alpha=7,D_alpha=6,S=9},x)); end, dix);');

% Convert the Maple resultant into a Matlab resultant

g=mapleresultant('dix2')

% Numerically solve the Matlab resultant

h=solveresultant(g)

% Check the solver but evaluating the resultant at one of the computed roots

x=evalresatx(g,h(7,1:2))
